#!/usr/bin/perl

######################################################################
#  Cognition/0.1-alpha11 - A metadata browser
#  Copyright (c) 2008 Toby Inkster.
######################################################################
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
######################################################################

package Cognition::Daemon;
use Net::Server::PreFork;
@ISA = qw(Net::Server::PreFork);

use strict;
use lib '.';
use Cache::FileCache;  # Causing problems?
use Cognition::HTMLParser;
use Cognition::Export::Contact;
use Cognition::Export::Feed;
use Cognition::Export::Location;
use Cognition::Export::Calendar;
use Data::Dumper;
use Getopt::Long
	qw(:config no_ignore_case bundling permute);
use HTTP::Cache::Transparent;
use HTTP::Request;
use JSON 2.0;
use LWP::UserAgent;
use Pod::Usage;

BEGIN { $ENV{PERL_JSON_BACKEND} = 'JSON::PP' }

HTTP::Cache::Transparent::init( { 
    BasePath  => "/tmp/cognition.cache",
    Verbose   => 1,
    MaxAge    => 4, # hours
    NoUpdate  => 5, # seconds
});

our $ObjectCache = new Cache::FileCache({
	'namespace' => 'Cognition.1',
	'cache_root' => '/tmp/cognition.objcache'
});

our $UA = LWP::UserAgent->new(
	agent => 'cognition/0.1-alpha11 ',
	from  => 'invalid@invalid.invalid'
	);

Cognition::Daemon->run(
	conf_file  => '/etc/cognition/cognitiond.conf'
);

sub options
{
	my $self     = shift;
	my $prop     = $self->{'server'};
	my $template = shift;
	
	$self->SUPER::options($template);
	
	$prop->{'format'} ||= undef;
	$template->{'format'} = \$prop->{'format'};

	foreach my $x (qw(erdf_strict_profiles grddl_fetch grddl_strict_profiles
		p_comments p_erdf p_grddl p_rdf p_rdfa p_rdfx p_structure p_uf
		rdfa_strict_doctype rdfa_strict_version uf_strict_profiles))
	{
		$prop->{opts}->{$x} ||= undef;
		$template->{$x} = \$prop->{opts}->{$x};
	}
}

sub process_request
{
	my $self = shift;
	my $autoclose = 0;
	my $output_format = $self->{server}->{'format'};
	
	my %opts = (ua => $UA);
	foreach my $k (keys %{ $self->{server}->{opts} })
		{ $opts{$k} = $self->{server}->{opts}->{$k}; }

	eval
	{
		local $SIG{ALRM} = sub { die "Timed Out!\n" };
		my $timeout = 60; # give the user 60 seconds to type a line
		my $previous_alarm = alarm($timeout);
		while( <STDIN> )
		{
			s/\r?\n$//;
			
			if (/^SET FORMAT (.+)$/i)
			{
				$output_format = $1;
				$self->log(4, "Set output format to '$1'.\n");
			}
			elsif (/^SET AUTOCLOSE (.+)$/i)
			{
				$autoclose = $1;
				$self->log(4, "Set autoclose to '$1'.\n");
			}
			elsif (/^SET OPTION (.+) (.+)$/i)
			{
				$opts{lc($1)} = $2;
				$self->log(4, "Set option $1 to '$2'.\n");
			}
			elsif (/^COGNIFY (.+)$/i)
			{
				my ($parser, $givenUrl, $pageUrl, $subjectUrl);
				$givenUrl = $1;
				$givenUrl = "http://localhost/~tai/$1.html"
					if ($givenUrl =~ /^T:(.*)$/);
					
				($pageUrl, $subjectUrl) = url_split($givenUrl);
				
				$self->log(2, "Cognifying '$subjectUrl' at page '$pageUrl'.\n");
				$parser = $self->get_parser($pageUrl, \%opts);

				print do_export($output_format, $parser, $subjectUrl);	
				
				return if ($autoclose);
			}
			elsif (/^VALIDATE (.+)$/i)
			{
				my ($parser, $givenUrl, $pageUrl, $subjectUrl);
				$givenUrl = $1;
				$givenUrl = "http://localhost/~tai/$1.html"
					if ($givenUrl =~ /^T:(.*)$/);
					
				($pageUrl, $subjectUrl) = url_split($givenUrl);
				
				$self->log(2, "Cognifying '$subjectUrl' at page '$pageUrl'.\n");
				$parser = $self->get_parser($pageUrl, \%opts);

				$self->log(2, "Doing validation.\n");
				print do_validate($parser);	
				
				return if ($autoclose);
			}
			elsif (/^QUIT$/i)
			{
				$self->log(4, "Client quit.\n");
				print "Bye!\r\n";
				return;
			}
			else
			{
				$self->log(0, "Huh? $_\n");
				print "Use 'COGNIFY http://example.org/foo.html'\r\n";
				return;
			}
			
			alarm($timeout);
		}
		alarm($previous_alarm);
	}
}

sub get_parser
{
	my $this = shift;
	my $url  = shift;
	my $o    = shift;
	my %opts = %$o;
	
	my $opt_string = '#';
	foreach my $k (sort keys %opts)
	{
		$opt_string .= $k.'='.$opts{$k}.'/' if (length $opts{$k} && $k =~ /\_/);
	}
	
	$opts{request}  = HTTP::Request->new(GET => $url);
	$opts{response} = $opts{ua}->request($opts{request});

	my ($parser, $found);
	if ($opts{response}->header('X-Content-Unchanged'))
	{
		$parser = $ObjectCache->get($url.$opt_string);
		$found  = defined $parser;
		$this->log(3, "Perfect cache hit for $url, opts $opt_string.")
			if ($found);
	}
	
	$parser = Cognition::HTMLParser->new($opts{response}->content, \%opts)
		unless ($parser);
		
	$ObjectCache->set($url.$opt_string, $parser, "20 minutes") unless ($found);
	return $parser;
}


sub url_split
{
	my $url = shift;
	
	if ($url =~ /^([^\#]+)\#subject\((.*)\)$/)
	{
		return ("$1", "$2");
	}
	elsif ($url =~ /^([^\#]+)\#(.*)$/)
	{
		return ("$1", "$1#$2");
	}
	else
	{
		return ($url, undef);
	}
}


sub do_export
{
	my ($output_format, $parser, $subjectUrl) = @_;

	if ($output_format =~ /PERL/)
	{
		return $parser->dumpText;
	}
	
	elsif ($output_format =~ /perl/i)
	{
		return $parser->dumpTextClean;
	}
	
	elsif ($output_format =~ /vcf/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Contact::outVCF($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Contact::multiVCF($parser->{RDF}); }
	}
	
	elsif ($output_format =~ /rdf.?json/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Entity::outJSON($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Entity::multiJSON($parser->{RDF}); }
	}
	
	elsif ($output_format =~ /kml/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Location::outKML($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Location::multiKML($parser->{RDF}); }
	}
	
	elsif ($output_format =~ /detect/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Entity::outDetect($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Entity::multiDetect($parser->{RDF}); }
	}
	
	elsif ($output_format =~ /jcard/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Contact::outJSON($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Contact::multiJSON($parser->{RDF}); }
	}
	
	elsif ($output_format =~ /atom/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Feed::outAtom($parser->{RDF}, $subjectUrl); }
		else
		{
			foreach my $subj (keys %{$parser->{RDF}})
			{
				if ( Cognition::Export::Feed::is($parser->{RDF}->{$subj}) )
				{
					return Cognition::Export::Feed::outAtom($parser->{RDF}, $subj);
				}
			}
		}
	}
	
	elsif ($output_format =~ /ics/i)
	{
		if ($subjectUrl)
			{ return Cognition::Export::Calendar::outICS($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Calendar::multiICS($parser->{RDF}); }
	}
	
	else
	{
		if ($subjectUrl)
			{ return Cognition::Export::Entity::outRDF($parser->{RDF}, $subjectUrl); }
		else
			{ return Cognition::Export::Entity::multiRDF($parser->{RDF}); }
	}
}


sub do_validate
{
	my $page = shift;
	my @rv;
	
	foreach my $hcard (@{ $page->{uF}->{hCard} })
	{
		my @errs = $hcard->validate;
		foreach my $e (@errs)
			{ push @rv, $e; }
	}
	
	my $x = {
		'validation'  => \@rv,
		'rdf'         => from_json(Cognition::Export::Entity::multiJSON($page->{RDF}), {utf8 => 1})
	};
	
	return to_json($x, {utf8 => 1, pretty => 1, canonical => 1, convert_blessed => 1})."\n";
}
